require 'fast_helper'
require 'background_jobs/job_queue_factory'
require 'background_jobs/strategies/sidekiq/job_queue'
require 'background_jobs/strategies/direct_call/job_queue'

module BackgroundJobs
  describe JobQueueFactory do
    describe '.build' do
      subject { JobQueueFactory.build }

      context 'sidekiq' do
        before { configure_strategy('sidekiq', options) }

        let(:sidekiq_job_queue) { stub('sidekiq_job_queue') }

        let(:options) { stub('options') }

        it { 
          Sidekiq::JobQueue.expects(:new).returns(sidekiq_job_queue)
          JobQueueFactory.build.should == sidekiq_job_queue 
        }

        after { clear_job_queue_strategy }
      end

      context 'direct_call' do
        before { configure_strategy('direct_call') }

        it { should be_instance_of DirectCall::JobQueue }

        after { clear_job_queue_strategy }
      end

      context 'with no strategy configured' do
        before { clear_job_queue_strategy  }

        it "will blow up" do
          expect {
            JobQueueFactory.build 
          }.to raise_exception UnknownBackgroundJobsQueueStrategy
        end

        after { configure_strategy('delayed_job') }
      end
    end
  end
end

def clear_job_queue_strategy
  configure_strategy nil
end

def configure_strategy(name, options = {})
  BackgroundJobs.configure do |config|
    config.strategy = name
    config.strategy_options = options
  end
end
