require 'fast_helper'
require 'time'
require 'background_jobs/strategies/sidekiq/job_queue'

module BackgroundJobs
  module Sidekiq
    describe JobQueue do
      let(:type) { :foo }
      let(:priority) { 1 }

      let(:scheduled_set) { stub('scheduled_set') }

      let(:entries) { [] }

      before do
        ::Sidekiq::ScheduledSet.stubs(:new).returns(scheduled_set)
        scheduled_set.stubs(:entries).returns(entries)

        BackgroundJobs.register_job type, :foo, Class.new
      end

      describe '.enqueue' do

        context 'with job not scheduled yet' do
          let(:entries) { [] }

          let(:time) { Time.parse('1st Jan 2013') }

          before do
            Time.stubs(:now).returns(time)
          end

          it 'should schedule the job if schedule is set' do
            ::Sidekiq::Client.expects(:push).with(
              'queue' => 'foo', 
              'class' => BackgroundJobs::Sidekiq::JobAdapter, 
              'args' => [type, 123, 'range:[10,10]', {'foo' => :bar}],
              'at' => (time + 600).to_i
            )

            do_enqueue
          end
        end

        context 'with job scheduled already' do
          let(:entries) {
            [stub('entry', :item => {'args' => [type.to_s]})]
          }

          it 'should not schedule any job' do
            ::Sidekiq::Client.expects(:push).never

            do_enqueue
          end
        end

        def do_enqueue
          JobQueue.new.enqueue(type, 123, [10..10, :foo => :bar], :in => 600, :unique => true)
        end
      end
    end
  end
end